@@ -1,3 +1,5 @@
+import pyvisa
+
 def get_resource(inputtype,devicetype,*arg):
     dev_list = []
     for i in range(1,20):
@@ -26,4 +28,8 @@ def identify(inputtype,devicetype,*arg):
             if count == 2: #set the search tolerance i.e this tries to communicate 2 times
                 identity = 'None'
         id_list.append(identity)
-    return id_list
\ No newline at end of file
+    return id_list
+
+def list_resources():
+    rm = pyvisa.ResourceManager()
+    return rm.list_resources()
\ No newline at end of file
